/*
 * Copyright (c) 1997 Manuel Bouyer.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *  This product includes software developed by Manuel Bouyer.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Librairie pour gerer les mots de passe dans /etc/passwd.pop.
 */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <errno.h>
#include <pwd.h>
#include <signal.h>
#include <unistd.h>
#include <varargs.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/file.h>
#include <time.h>
#include "lib.h"

#define PF_PLATE        "./ptmpXXXXX"   /* original temporary file */
#define PF_TEMP         "./ptmp"        /* lock file */

/*
 * library functions
 */

#ifdef sun
extern int mkstemp();                  /* makes a temp file name */
extern char *crypt();
extern int errno;
extern char *sys_errlist[];
#endif

void makesalt __P((char c[]));
void sigoff __P(());
void sigon __P(());

/*
 * global variables
 */
char *pf_name = "/etc/passwd.pop";      /* password file name */
char *pf_tnam;                          /* name of temporary file */
FILE *pf_fp;                            /* pointer to real password file */
FILE *pf_tfp;                           /* pointer to temporary file */
FILE *log;
time_t gNow;
char gDate[BUFSIZE];


void UpdateTime( )
{
        struct tm *date;
        
        gNow = time( NULL );            /* get current calendar time */
        date = localtime( &gNow );      /* convert it to a broken-down time */
        strftime( gDate, BUFSIZE, "%b %d %H:%M:%S", date );
}

void WriteLog( va_alist )
va_dcl
{
        va_list ap;
        char *fmt;      
        char *args[MAXARGS];
        int argno = 0;
        UpdateTime( );


        va_start( ap );
        fmt = va_arg(ap, char *);
        while ((args[argno++] = va_arg(ap, char *)) != (char *)0)
                    ;
        fprintf( log, "%s: ", gDate );
        vfprintf(log, fmt, args);
        fputs( "\n", log );
        fflush(log);
        va_end( ap );
}

int chkPass( user, pass, pw )
char *user;
char *pass;
struct passwd *pw;
{
        /*  Compare the supplied password with the password file entry */
        if (strcmp (crypt (pass, pw->pw_passwd), pw->pw_passwd) != 0)
                return ( FAILURE );
        else 
                return ( SUCCESS );
}




void makesalt( c )
char c[2];                      /* salt characters */
{
        register long salt;             /* used to compute a salt */
        register int i;                 /* counter in a for loop */

        /*
         * just mix a few things up for the salt ...
         * no rhyme or reason here
         */
        salt = (((long) time(TI_NULL))&0x3f) | (getpid() << 5);

        /*
         * use the bottom 12 bits and map them into the legal alphabet
         */
        for(i = 0; i < 2; i++){
                c[i] = (salt & 0x3f) + '.';
                if (c[i] > '9')
                        c[i] += 7;
                if (c[i] > 'Z')
                        c[i] += 6;
                salt >>= 6;
        }
}


int setPass( user, pass, newpass, pw )
char *user;
char *pass;
char *newpass;
struct passwd *pw;
{

        char saltc[2];          /* the password's salt */

        makesalt(saltc);
        strcpy( pw->pw_passwd, crypt(newpass, saltc));

        if ( UpdatePasswordFile( pw ) == PW_CHANGED )
                return (SUCCESS);
        else
                return (FAILURE);
}



int UpdatePasswordFile(p)
struct passwd *p;               /* new password structure */
{
        static char nbuf[BUFSIZ*2];     /* buffer for passwords being read */
        static char obuf[BUFSIZ*2];     /* buffer for new entry */
        register char *q, *s;           /* used for quick name comparison */
        int retval = PW_NOCHANGE;       /* return value */
		int tmpf;						/* temp file */

        /*
         * disable ALL signals at this point
         */
        sigoff();

        /*
         * open the temporary password file
         */
		/* We have to copy the string, because gcc store strings in
		 * read-only memory and mkstemp changes it
		 */
		pf_tnam = malloc(strlen(PF_PLATE)+1);
		strcpy(pf_tnam, PF_PLATE);
        umask(022);
        tmpf = mkstemp(pf_tnam);
        if ((pf_tfp = fdopen(tmpf, "w")) == FI_NULL){
				WriteLog("open %s failed: %s\n", pf_tnam, sys_errlist[errno]);
                retval = PW_CANTOPEN;
                goto cantlock;
        }

        /*
         * lock the password file
         */
        if (link(pf_tnam, PF_TEMP) < 0){
				WriteLog("link %s to %s failed: %s\n", pf_tnam,
					PF_TEMP, sys_errlist[errno]);
                retval = PW_CANTLOCK;
                goto cantlock;
        }
        
        /*
         * copy the new password structure
         */
        sprintf(obuf, "%s:%s\n", p->pw_name, p->pw_passwd);

        /*
         * open the password file
         */
        if ((pf_fp = fopen(pf_name, "r")) == FI_NULL){
				WriteLog("open %s failed: %s\n", pf_name, sys_errlist[errno]);
                retval = PW_CANTOPEN;
                goto getout;
        }


        /*
         * copy the password file into the temporary one
         */
        while(fgets(nbuf, BUFSIZ, pf_fp) != NULL){
                for(s = nbuf, q = p->pw_name; *s && *s != ':'; s++, q++)
                        if (*s != *q)
                                break;
                if (*s == ':' && *q == '\0') {
                        fputs(obuf, pf_tfp);
                        retval = PW_CHANGED;
                } else {
                        fputs(nbuf, pf_tfp);
				}
        }
        if (retval == PW_NOCHANGE) {
                goto getout;
        }

        /*
         * close the temporary file and the real one
         */
        (void) fclose(pf_tfp);
        pf_tfp = FI_NULL;
        (void) fclose(pf_fp);
        pf_fp = FI_NULL;

        /*
         * now relink; note the lock file is still there
         */
        if ( unlink(pf_name) >= 0
                && link(pf_tnam, pf_name) >= 0
                && unlink(pf_tnam) >= 0){
                retval = PW_CHANGED;
        } else {
				WriteLog("unlink/link failed: %s\n", sys_errlist[errno]);
                retval = PW_CANTRENAME;
		}

getout:
        /*
         * Only remove lock file if this program obtained it
         */
         (void) unlink(PF_TEMP); 

cantlock:
        /* some clean up */
        if (pf_tfp != FI_NULL)
                (void) fclose(pf_tfp);
        if (pf_fp != FI_NULL)
                (void) fclose(pf_fp);
        (void) unlink(pf_tnam);

        sigon(); /* re-enable ALL signals at this point */

        return(retval);
}


/*
 * signal handling routines
 */
void (*trap[NSIG])();           /* values returned by signal() */


/*
 * disable ALL signal trapping
 */
void sigoff()
{
        register int i;         /* counter in a for loop */

        for(i = 0; i < NSIG; i++)
                (void) signal(i, SIG_IGN);
}

/*
 * restore to the initial setting all signals
 */
void sigon()
{
        register int i;         /* counter in a for loop */

        for(i = 0; i < NSIG; i++)
                (void) signal(i, trap[i]);

}

struct passwd *
getpoppwnam(user)
	char *user;

{
		FILE *poppwd;
		static char pwbuff[BUFSIZE * 2];
		char *name, *passwd, *c;
		static struct passwd pw;

    if ((poppwd = fopen(pf_name, "r")) == NULL) {
       	return 0;
   	}
    while (feof(poppwd) == 0) {
		fgets(pwbuff, BUFSIZE, poppwd);
		name = pwbuff;
		for (c = pwbuff ; *c != ':' && *c != '\0'; c++);
		*c = '\0'; c++;
		passwd = c;
		for (; *c != '\n'; c++); /* Strip "\n" at the end of entry */
		*c = '\0';
		if (strcmp(name, user) == 0) {
			fclose(poppwd);
		    pw.pw_name = name;
		    pw.pw_passwd = passwd;
		    return &pw;
		}
	}
	fclose(poppwd);
	return 0;
}
