/*
 * Copyright (c) 1997 Manuel Bouyer.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *  This product includes software developed by Manuel Bouyer.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * poppasswd.c Update passwd from the command line in the file /etc/passwd.pop
 */

#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <signal.h>
#include <unistd.h>
#include <varargs.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/file.h>
#include <time.h>
#include "lib.h"

#ifdef sun
char *getpass __P((char*));
#endif

main (argc, argv)
int argc;
char *argv[];
{

		static char user[BUFSIZE];
		static char pass[BUFSIZE];
		static char newpass[BUFSIZE];
		char *tmpstr;
        struct passwd *pw;

		chdir( "/etc" );

        /*
         * Set the log file to stderr
         */

		 log = stderr;

		if (argc > 2 || (argc == 2 && *argv[1] == '-')) {
			fprintf(stderr, "usage: %s [username]\n", argv[0]);
			exit(1);
		}

		if (argc == 2) {
			if (getuid() != 0) {
				fprintf(stderr, "%s: permission denied\n", argv[0]);
				exit(1);
			}
			strncpy(user, argv[1], BUFSIZE);
			user[BUFSIZE-1]  = '\0';
		} else { /* get name from uid */
			pw = getpwuid(getuid());
			strncpy(user, pw->pw_name, BUFSIZE);
			user[BUFSIZE-1]  = '\0';
		}

        /* test for valid user */
        if ( (pw=getpoppwnam(user)) == NULL ) {
                fprintf(stderr,"Unknon user %s\n", user);
                exit(1);
        }

       	printf("changing POP passwd for %s.\n", user); 
		if (getuid() != 0) {
			tmpstr = getpass("Old password:");
			if (tmpstr == NULL)
				exit(0);
			strncpy(pass, tmpstr, BUFSIZE);
			pass[BUFSIZE-1]  = '\0';
        	/* authenticate user */
        	if ( chkPass( user, pass, pw ) == FAILURE ) {
                	fprintf(stderr,"Permission denied\n");
                	exit(1);
        	}
		}

again:	tmpstr = getpass("New password:");
		if (tmpstr == NULL)
			exit(0);
		strncpy(newpass, tmpstr, BUFSIZE);
		newpass[BUFSIZE-1]  = '\0';

        /* new pass must be 6 char or longer */
        if ( strlen(newpass) < 6 ) {
                fprintf(stderr, "Password must have at last 6 characters\n");
                goto again;
        }
        tmpstr = getpass("New password (again):");
		if (tmpstr == NULL)
			exit(0);
		if (strcmp(newpass, tmpstr) !=  0) {
			fprintf(stderr, "passord mismatch\n");
			goto again;
		}
        if ( setPass( user, pass, newpass, pw ) == FAILURE ) { 
                fprintf(stderr,"Unable to change password.\n" );
                exit(1);
        }

        fprintf(stderr, "POP password changed\n");
        exit(0);
}
